//    WeIRCd IRC server software - sub file
//    Copyright (C) 2007-2009 Jonas Thiem
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License v3 as published
//    by the Free Software Foundation.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.

#include "weircd.h"
#ifndef WIN
#include <dirent.h>
#endif

int isdir(const char* path) {
#ifndef WIN
	struct stat info;
	int r = stat(path,&info);
	if (r < 0) {return -1;}
	if (S_ISDIR(info.st_mode) != 0) {return 0;}
	return -1;
#endif
#ifdef WIN
	WIN32_FIND_DATA findFileData;
	ZeroMemory(&findFileData, sizeof(WIN32_FIND_DATA));
	HANDLE hf = FindFirstFile(path, &findFileData);
	if (hf == INVALID_HANDLE_VALUE) {return -1;}
	FindClose(hf);
	if ((findFileData.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY) != 0) {
		return 0;
	}
	return -1;
#endif
}

char tellfileerror_buffer[500];
void tellfileerror_setstaticbuffer() {
	char* p = tellfileerror_buffer;
#ifndef WIN
	if (errno == EPERM || errno == EACCES) {
		strcpy(p,"no permission");return;
	}
	if (errno == ENOENT || errno == EFAULT || errno == ENODEV) {
		strcpy(p,"file not found!");return;
	}
	if (errno == EISDIR) {
		strcpy(p,"target is a directory");return;
	}
	if (errno == EBUSY) {
		strcpy(p,"device is busy");return;
	}
	if (errno == ETXTBSY) {
		strcpy(p,"blocked because of other read/write operation (likely of another program)");return;
	}
	if (errno == EMFILE || errno == ENFILE) {
		strcpy(p,"too many files are already opened");return;
	}
#endif
#ifdef WIN
	int err = GetLastError();
	if (err == ERROR_ACCESS_DENIED || err == ERROR_INVALID_ACCESS || err == ERROR_WRITE_PROTECT) {
		strcpy(p,"no permission");return;
	}
	if (err == ERROR_FILE_NOT_FOUND || err == ERROR_PATH_NOT_FOUND) {
		strcpy(p,"file not found!");return;
	}
	if (err == ERROR_NOT_READY || err == ERROR_WRITE_FAULT || err == ERROR_SHARING_BUFFER_EXCEEDED || err == ERROR_HANDLE_DISK_FULL || err == ERROR_NETWORK_BUSY || err == ERROR_OUT_OF_STRUCTURES) {
		strcpy(p,"device is busy");return;
	}
	if (err == ERROR_SHARING_VIOLATION || err == ERROR_LOCK_VIOLATION) {
		strcpy(p,"blocked because of other read/write operation (likely of another program)");return;
	}
	if (err == ERROR_TOO_MANY_OPEN_FILES) {
		strcpy(p,"too many files are already opened");return;
	}
#endif
	strcpy(p,"unknown error");
	return;
}
const char* tellfileerror() {
	tellfileerror_setstaticbuffer();
	return tellfileerror_buffer;
}