//    WeIRCd GUI Control
//    Copyright (C) 2009 Shahriar Heidrich, Jonas Thiem
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License v3 as published
//    by the Free Software Foundation.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.

#define MAINWNDID 1000

#define BTNSTARTID 1001
#define BTNSTOPID 1002
#define BTNREHASHID 1003
#define BTNCONFIGID 1004

HBITMAP logob = NULL;

const char *g_WeircdVersion = "WeIRCd 0.6.0.3"; //we need to obtain this in real time later.

// Functions to be filled with actual content
void OnClickedStart () {};
void OnClickedStop () {};
void OnClickedRehash () {};
void OnClickedConfig () {};

// Declaration of wndproc callback
LRESULT CALLBACK WindowProc(HWND Wnd, UINT Msg,
	WPARAM WParam, LPARAM LParam);

// Button handles
HWND Button_Start;
HWND Button_Stop;
HWND Button_Status;
HWND Button_Rehash;
HWND Button_Config;

HINSTANCE minst;
int gwinapi_show(HINSTANCE Inst) {
	HWND Text_Version;
	// WinAPI variables
	HWND MainWnd;
	minst = Inst; //save the instance globally accessible for the whole winapi functions in this file
	
	HFONT MainFont; //our custom font
	//create the font:
	MainFont = CreateFont(
	         0, 0, 0, 0,
	         FW_LIGHT,
	         FALSE,
	         FALSE,
	         FALSE,
	         ANSI_CHARSET,
	         OUT_DEFAULT_PRECIS,
	         CLIP_DEFAULT_PRECIS,
	         DEFAULT_QUALITY,
	         FF_DONTCARE,
	         NULL);
	
	// some constant sizes used to construct the GUI
	const int border = 5;
	const int smallbw = 100;
	const int ButtonW = 2*smallbw+border;
	const int ButtonH = 25;
	const int ButtonD = 2;
	const int freetopspace = 65;
	const int windowh = 160+freetopspace;
	int windoww = ButtonW+2*border+6;
	
	int CurButton = 0;
	
	const char *WndClassName = "WeIRCd Control Window";
	
	WNDCLASSEX MainWndCl;
	
	MainWndCl.cbSize = sizeof(WNDCLASSEX);
	MainWndCl.style = CS_HREDRAW | CS_VREDRAW;
	MainWndCl.lpfnWndProc = WindowProc;
	MainWndCl.cbClsExtra = 0;
	MainWndCl.cbWndExtra = 0;
	MainWndCl.hInstance = Inst;
	MainWndCl.lpszClassName = WndClassName;
	MainWndCl.lpszMenuName = NULL;
	MainWndCl.hIcon   = LoadIcon (NULL, IDI_WINLOGO);
	MainWndCl.hIconSm = LoadIcon (NULL, IDI_WINLOGO);
	MainWndCl.hCursor = LoadCursor (NULL, IDC_ARROW);
	HBRUSH MyBrush = CreateSolidBrush( RGB( 216, 213, 201 ) );
	MainWndCl.hbrBackground = MyBrush;
	
	if (RegisterClassEx(&MainWndCl) == 0)
	{
		printf ("Fail.");
		return -1;
	}
	
	MainWnd = CreateWindowEx(0, WndClassName,
		"WeIRCd Control", // title 
		WS_OVERLAPPED | WS_MINIMIZEBOX | WS_CAPTION | WS_SYSMENU, // flags
		CW_USEDEFAULT, // x
		CW_USEDEFAULT, // y
		windoww, // w
		windowh, // h
		(HWND) NULL, (HMENU) NULL,
		Inst, NULL);
	
	// Check for errors
	if (MainWnd == 0)
	{
		printf ("Couldn't initialize window.\n");
		printf ("Make sure your system is actually working.\n");
		return -1;
	}
	
	Button_Start = CreateWindow("BUTTON",
	"Start",
	WS_BORDER | WS_CHILD | WS_VISIBLE,
	border, freetopspace+(ButtonH+ButtonD)*CurButton,
	smallbw, ButtonH,
	MainWnd,
	(HMENU) BTNSTARTID,
	Inst, NULL);
	
	//CurButton++;
	
	Button_Stop = CreateWindow("BUTTON",
	"Stop",
	WS_BORDER | WS_CHILD | WS_VISIBLE,
	border+smallbw+border, freetopspace+(ButtonH+ButtonD)*CurButton,
	smallbw, ButtonH,
	MainWnd,
	(HMENU) BTNSTOPID,
	Inst, NULL);
	
	CurButton++;
	
	Button_Status = CreateWindow("BUTTON",
	"Retrieve status details",
	WS_BORDER | WS_CHILD | WS_VISIBLE,
	border, freetopspace+(ButtonH+ButtonD)*CurButton,
	ButtonW, ButtonH,
	MainWnd,
	(HMENU) NULL,
	Inst, NULL);
	
	CurButton++;
	
	Button_Config = CreateWindow("BUTTON",
	"Read/Edit the config file",
	WS_BORDER | WS_CHILD | WS_VISIBLE,
	border, freetopspace+(ButtonH+ButtonD)*CurButton,
	ButtonW, ButtonH,
	MainWnd,
	(HMENU) BTNCONFIGID,
	Inst, NULL);
	
	CurButton++;
	
	Button_Rehash = CreateWindow("BUTTON",
	"Reload the config file",
	WS_BORDER | WS_CHILD | WS_VISIBLE,
	border, freetopspace+(ButtonH+ButtonD)*CurButton,
	ButtonW, ButtonH,
	MainWnd,
	(HMENU) BTNREHASHID,
	Inst, NULL);
	
	CurButton++;
	
	// Create version info text
	Text_Version = CreateWindow("STATIC",
	g_WeircdVersion,
	WS_BORDER | WS_CHILD | WS_VISIBLE,
	0, windowh-48,
	windoww-6, 20,
	MainWnd,
	(HMENU) 0,
	Inst, NULL);
	
	// Set other font (needs to be done for every child object separately)
	SendMessage (Button_Start, WM_SETFONT, (WPARAM)MainFont, TRUE);
	SendMessage (Button_Stop, WM_SETFONT, (WPARAM)MainFont, TRUE);
	SendMessage (Button_Status, WM_SETFONT, (WPARAM)MainFont, TRUE);
	SendMessage (Button_Config, WM_SETFONT, (WPARAM)MainFont, TRUE);
	SendMessage (Button_Rehash, WM_SETFONT, (WPARAM)MainFont, TRUE);
	SendMessage (Button_Start, WM_SETFONT, (WPARAM)MainFont, TRUE);
	SendMessage (Text_Version, WM_SETFONT, (WPARAM)MainFont, TRUE);
	
	// Disable buttons as WeIRCd is still turned off FIXME: we do not know that yet!
	EnableWindow(Button_Stop, FALSE);
	EnableWindow(Button_Status, FALSE);
	EnableWindow(Button_Rehash, FALSE);
	
	// Display window
	ShowWindow(MainWnd, SW_SHOWDEFAULT);
	UpdateWindow(MainWnd);
}

int gwinapi_process() {
	MSG Message;
	// check messages
	if (GetMessage (&Message, NULL, 0, 0)) {
		TranslateMessage (&Message);
		DispatchMessage (&Message);
		return 0;
	}else{
		return -1;
	}
}

// WndProc
LRESULT CALLBACK WindowProc(HWND Wnd, UINT Msg,
	WPARAM WParam, LPARAM LParam) {

	switch (Msg) {
	
		case WM_COMMAND: {
			switch (WParam) {
			
				case BTNSTARTID: {
					OnClickedStart();break;
				} 
				case BTNSTOPID: {
					OnClickedStop();break;
				}
				case BTNREHASHID: {
					OnClickedRehash();break;
				}
				case BTNCONFIGID: {
					OnClickedConfig();break;
				}
			}
			break;
		}
		case WM_CREATE: {
			logob = LoadBitmap(minst, "IDI_LOGO");
			if (logob == NULL) {
				MessageBox(Wnd, "Failed to load logo bitmap ressource", "Nonfatal error", MB_OK | MB_ICONEXCLAMATION);
			}
			break;
		}
		case WM_PAINT: {
			BITMAP bm;
			PAINTSTRUCT ps;

			HDC hdc = BeginPaint(Wnd, &ps);

			HDC hdcMem = CreateCompatibleDC(hdc);
			HBITMAP hbmOld = SelectObject(hdcMem, logob);

			GetObject(logob, sizeof(bm), &bm);

			BitBlt(hdc, 10, 5, bm.bmWidth, bm.bmHeight, hdcMem, 0, 0, SRCCOPY);

			SelectObject(hdcMem, hbmOld);
			DeleteDC(hdcMem);

			EndPaint(Wnd, &ps);
			break;
		}
		case WM_DESTROY: {
			PostQuitMessage (0);
			return 0;
		}
	
	}
	return DefWindowProc(Wnd, Msg, WParam, LParam);
}
