//    WeIRCd IRC server software - sub file
//    Copyright (C) 2007-2009 Jonas Thiem
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License v3 as published
//    by the Free Software Foundation.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.

#include "weircd.h"

#define IPREMOVESECONDS (60*30)

void* iplist = NULL;
void* ips_GetPtr(const char* vIp) {
	void* item = iplist;
	while (item != NULL) {
		struct weircd_ipdata* w = linkedlist_GetData(item);
		if (strcasecmp(w->ip,vIp) == 0) {
			return item;
		}
		item = linkedlist_NextItem(item);
	}
	return NULL;
}
void* ips_Add(const char* vIp) {
	void* r = ips_GetPtr(vIp);
	if (r != NULL) {return r;}
	struct weircd_ipdata w;
	memset(&w,0,sizeof(w));
	strcpy(w.ip,vIp);
	return linkedlist_Append(&iplist,&w,sizeof(w));
}
struct weircd_ipdata* ips_GetDataByPtr(void* ptr) {
	return linkedlist_GetData(ptr);
}
int ips_ClonesAlarm(const char* vIp) {
	if (config_IntVal("MaxConnsPerIp") <= 0) {return -1;}
	void* r = ips_GetPtr(vIp);
	if (r == NULL) {
		r = ips_Add(vIp);
		if (r == NULL) {return -1;}
	}
	int k = 0;
	usr_CycleInit();
	r = usr_DoCycle();
	while (r != NULL) {
		if (strcasecmp(udat(r)->ip,vIp) == 0) {
			k += 1;
		}
		r = usr_DoCycle();
	}
	if (k > config_IntVal("MaxConnsPerIp")) {
		usr_CycleInit();
		r = usr_DoCycle();
		while (r != NULL) {
			if (strcasecmp(udat(r)->ip,vIp) == 0) {
				usnd(r,"ERROR :Closing link: Clone limit reached");
				userQuit(r,USERQUIT_ERROR);
			}
			r = usr_DoCycle();
		}
		return 0;
	}
	return -1;
}

int ips_Throttled(const char* vIp) {
	void* r = ips_GetPtr(vIp);
	struct weircd_ipdata* w;
	if (r == NULL) {
		r = ips_Add(vIp);
		if (r != NULL) {
			w = ips_GetDataByPtr(r);
			w->lastconnect2 = ts();
		}
		return -1;
	}
	w = ips_GetDataByPtr(r);
	if (w->lastconnect + 60 > w->lastconnect2 && w->lastconnect2 + 20 > ts()) {
		return 0;
	}
	w->lastconnect = w->lastconnect2;
	w->lastconnect2 = ts();
	return -1;
}
long long lastcheck = 0;
void ips_AutoRemove() {
	if (lastcheck >= ts()-1) {return;}
	lastcheck = ts();
	int removedone = 0;
	while (removedone == 0) {
		removedone = 1;
		void* item = iplist;
		while (item != NULL) {
			struct weircd_ipdata* w = linkedlist_GetData(item);
			void* r = NULL;
			int found = 0;
			usr_CycleInit();
			r = usr_DoCycle();
			while (r != NULL) {
				if (strcasecmp(udat(r)->ip,w->ip) == 0) {
					found = 1;break;
				}
				r = usr_DoCycle();
			}
			if (found == 1) {
				w->lastused = ts();
			}else{
				if (w->lastused + IPREMOVESECONDS < ts()) {
					//expired item; remove it
					linkedlist_RemoveItem(&iplist,item);
					removedone = 0;
					break;
				}
			}
			item = linkedlist_NextItem(item);
		}
	}
}
int ips_NickservKill(const char* ip) {
	void* r = ips_GetPtr(ip);
	struct weircd_ipdata* w;
	if (r == NULL) {
		r = ips_Add(ip);
		if (r == NULL) {return -1;}
	}
	w = ips_GetDataByPtr(r);
	w->lastnickfail3 = w->lastnickfail2;
	w->lastnickfail2 = w->lastnickfail1;
	w->lastnickfail1 = ts();
	if (w->lastnickfail1 + 60 > ts() && w->lastnickfail2 + 120 > ts() && w->lastnickfail3 + 120 > ts()) {
		w->lastnickfail1 = 0;w->lastnickfail2 = 0;w->lastnickfail3 = 0;
		return 0;
	}
	return -1;
}
int ips_RenameBlocked(const char* ip) {
	//this gets called when a rename is done shortly after another
	// -- and it will issue a nick lock so that any following nick change
	//    in a specific time window gets blocked
	void* r = ips_GetPtr(ip);
	struct weircd_ipdata* w;
	if (r == NULL) {
		r = ips_Add(ip);
		if (r == NULL) {return -1;}
	}
	w = ips_GetDataByPtr(r);
	
	if (w->nicklock > ts()) {
		return 0;
	}else{
		w->nicklock = ts()+20;
	}
	return -1;
}
void ips_BlockRename(const char* ip) {
		void* r = ips_GetPtr(ip);
	struct weircd_ipdata* w;
	if (r == NULL) {
		r = ips_Add(ip);
		if (r == NULL) {return -1;}
	}
	w = ips_GetDataByPtr(r);
	w->nicklock = ts()+20;
}
void ips_NickservLogin(const char* ip) {
	void* r = ips_GetPtr(ip);
	struct weircd_ipdata* w;
	if (r == NULL) {
		return;
	}
	w = ips_GetDataByPtr(r);
	w->lastnickfail3 = 0;
	w->lastnickfail2 = 0;
	w->lastnickfail1 = 0;
}