//    WeIRCd IRC server software - sub file
//    Copyright (C) 2007-2009 Jonas Thiem
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License v3 as published
//    by the Free Software Foundation.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.

#include "weircd.h"

#ifdef WIN
const char modext[] = ".dll";
#endif

#ifndef WIN
#ifndef MAC
const char modext[] = ".so";
#endif
#ifdef MAC
const char modext[] = ".dylib";
#endif
#endif

const char* modbreakage[] = {"", "Failed to load module", "Module misses info query function", "Module reports invalid name", "Module reports name twice", "Memory allocation failed", "Module reports bogus version", "Module doesn't report a name",
"Module reports version twice", "Module has bogus author info", "Module reports authors twice", "Module has bogus comment", "Module reports multiple comments", "Bogus module has no version info", "Bogus module reports functionality twice", "Bogus module doesn't provide needed module_FakeUserRecv()", "Bogus module triggered access violation", "Bogus module misses vital functions [services api]" };

int modloaderror = 0;

int module_ModLoadError() {
	return modloaderror;
}
struct moduleinfo staticinf;
struct moduleinfo* module_QueryInfo_NoUnload(const char* modname, void** ptr, int* dounload) {
	/* Do an info query and don't unload the module (if nothing responded was terribly wrong) */
	*dounload = 0;
	weircdapi_PrepareInfoQuery(modname);
	void* p = modman_GetPtrByName(modname);
	if (p == NULL) {
		if (modman_Load(modname) != 0) {
			modloaderror = MODULEFAIL_LoadingFailed;
			*ptr = NULL;*dounload = 0;
			return NULL;
		}
		p = modman_GetPtrByName(modname);
		if (p == NULL) {
			modman_Unload(modname);
			modloaderror = MODULEFAIL_LoadingFailed;
			*ptr = NULL;*dounload = 0;
			return NULL;
		}
		*dounload = 1;
		*ptr = p;
	}else{
		*dounload = 0;
		*ptr = p;
	}
	voidfunc queryfunc = modman_GetData(modname,"module_InfoQuery");
	if (queryfunc == NULL) {
		modloaderror = MODULEFAIL_NoInfoQuery;
		modman_Unload(modname);
		return NULL;
	}
	queryfunc();
	weircdapi_CloseInfoQuery();
	if (weircdapi_IsModuleBroken() > 0) {
		modloaderror = weircdapi_IsModuleBroken();
		if (*dounload == 1) {
			modman_Unload(modname);
		}else{
			modlist_SafeShutdown(modname);
		}
		*ptr = NULL;
		*dounload = 0;
		return NULL;
	}
	if (module_AccessViolationHappened() == 0) {
		modloaderror = MODULEFAIL_AccessViolation;
		if (*dounload == 1) {
			modman_Unload(modname);
		}else{
			modlist_SafeShutdown(modname);
		}
		*ptr = NULL;
		*dounload = 0;
		return NULL;
	}
	module_Switched(NULL);
	const char* currentname = weircdapi_GetName();
	memset(&staticinf,0,sizeof(staticinf));
	staticinf.name = malloc(strlen(currentname)+1);
	if (staticinf.name == NULL) {
		modloaderror = MODULEFAIL_MallocFailed;
		if (*dounload == 1) {
			modman_Unload(modname);*ptr = NULL;
		}
		*dounload = 0;
		return NULL;
	}
	strcpy(staticinf.name,currentname);
	weircdapi_SetInfo(&staticinf);
	return &staticinf;
}
struct moduleinfo* module_QueryInfo(const char* modname) {
	int dounload;
	void* ptr;
	struct moduleinfo* retval = module_QueryInfo_NoUnload(modname,&ptr,&dounload);
	if (dounload == 1) {
		modman_Unload(modname);
	}
	return retval;
}
void module_FreeInfo(struct moduleinfo* inf) {
	if (inf->name != NULL) {
		free(inf->name);
		inf->name = NULL;
	}
	if (inf->authors != NULL) {
		free(inf->authors);
		inf->authors = NULL;
	}
	if (inf->comment != NULL) {
		free(inf->comment);
		inf->comment = NULL;
	}
}
#ifndef WIN
#include <dirent.h>
#endif

int printmodulelist(USER userid, const char* format) {
	const char* nick;
	int c = 0;
	if (userid != NOUSER) {
		nick = udat(userid)->nickname;
	}
#ifndef WIN
	struct dirent **namelist;
	int n;
	n = scandir(moduledir, &namelist, 0, alphasort);
	if (n < 0) {
		return 0;
	}else {
		while(n--) {
			const char* p = namelist[n]->d_name;
			if (strcasecmp(p,".") != 0 && strcasecmp(p,"..") != 0) {
				if (isdir(p) != 0 && strendswith(p,modext) == 0) {
					c++;
					if (userid != NOUSER && format != NULL) {
						char* cpy = alloca(strlen(p)+1);
						strcpy(cpy,p);
						cpy[strlen(cpy)-strlen(modext)] = 0;
						usnd(userid,format,nick,cpy);
					}else{
						printf("%s\n",p);
					}
				}
			}
			free(namelist[n]);
		}
		free(namelist);
	}
#endif
#ifdef WIN
	WIN32_FIND_DATA ffd;
	HANDLE hFind = INVALID_HANDLE_VALUE;
	char sep[] = "/";if (strendswith(moduledir,"/") == 0 || strlen(moduledir) <= 0) {strcpy(sep,"");}
	char finddir[200];snprintf(finddir,199,"%s%s*",moduledir,sep);finddir[199] = 0;
	hFind = FindFirstFile(finddir, &ffd);
	if (INVALID_HANDLE_VALUE == hFind) {
		return 0;
	}
	while (1) {
		const char* p = ffd.cFileName;
		if (strcasecmp(p,".") != 0 && strcasecmp(p,"..") != 0) {
			if (isdir(p) != 0 && strendswith(p,modext) == 0) {
				c++;
				if (userid != NOUSER && format != NULL) {
					char* cpy = alloca(strlen(p)+1);
					strcpy(cpy,p);
					cpy[strlen(cpy)-strlen(modext)] = 0;
					usnd(userid,format,nick,cpy);
				}else{
					printf("%s\n",p);
				}
			}
		}
		if (FindNextFile(hFind, &ffd) == 0) {break;}
	}
	FindClose(hFind);
#endif
	return c;
}