//    weircdservices module
//    Copyright (C) 2009 Jonas Thiem
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License v3 as published
//    by the Free Software Foundation.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.

#include "../weircd_api.h"
#include "../weircd_api_fakeusers.h"
#include "../weircd_api_services.h"

//#include "md5.h"
char md5result[50];
#include "md5.c"

/* Preface: some options */

char host[200];
int nicknameregisterwaittime;
int dolimitchanchars;
int forcerenameseconds;

void privmsg_nickserv(const char* user, const char* msg);
void privmsg_chanserv(const char* user, const char* msg);
void privmsg_operserv(const char* user, const char* msg);
int opendbs();
void closedbs();

/* Section 1: InfoQuery function providing info about the module*/

int major = 1;
int minor = 0;
int release = 0;
void module_InfoQuery() {
	weircd_infoq_ReportName("WeIRCd Services");
	weircd_infoq_ReportVersion(major,minor,release);
	weircd_infoq_ReportComment("The official built-in services");
	weircd_infoq_ReportAuthor("Jonas Thiem");
	weircd_infoq_ReportServicesFunctionality();
}

/* Section 2: pre parsing config options */

int optionnickwaittime;
int optionhost;
int optionlimitchanchars;
int optionforcerename;
void module_ServicesCOptionPreCheck_Begin() {
	//reset the option switches (to indicate no options have been set yet)
	optionnickwaittime = -1;
	optionhost = 0;
	optionlimitchanchars = 0;
	optionforcerename = 0;
}
int module_ServicesCOptionPreCheck(const char* option, const char* value) {
	//evaluate the config option and return 0 on success
	//set the option switch of that option to 1 to indicate it has been set already
	//printf("PRECHECK %s=>%s\n",option,value);
	if (strcasecmp(option,"host") == 0) {
		if (optionhost == 1) {return -1;}
		if (strlen(value) <= 0 || strlen(value) >= 130) {return -1;}
		if (charpos(value,' ') >= 0) {return -1;}
		if (charpos(value,'!') >= 0) {return -1;}
		if (charpos(value,'@') >= 0) {return -1;}
		if (charpos(value,',') >= 0) {return -1;}
		optionhost = 1;
		return 0;
	}
	if (strcasecmp(option,"forcerename") == 0) {
		if (optionforcerename == 1) {return -1;}
		if (isnumber(value) != 0) {return -1;}
		if (strlen(value) < 0 || strlen(value) > 6) {return -1;}
		if (atoi(value) < 0) {return -1;}
		optionforcerename = 1;
		return 0;
	}
	if (strcasecmp(option,"nickwaittime") == 0) {
		if (optionnickwaittime == 1) {return -1;}
		if (isnumber(value) != 0) {return -1;}
		if (strlen(value) < 0 || strlen(value) > 6) {return -1;}
		if (atoi(value) < 0) {return -1;}
		optionnickwaittime = 1;
		return 0;
	}
	if (strcasecmp(option,"limitchanchars") == 0) {
		if (optionlimitchanchars == 1) {return -1;}
		if (strcasecmp(value,"true") != 0 && strcasecmp(value,"false") != 0) {
			return -1;
		}
		optionlimitchanchars = 1;
		return 0;
	}
	return -1;
}
int module_ServicesCOptionPreCheck_End() {
	//check if config options are missing
	if (optionnickwaittime < 0 || optionhost == 0 || optionlimitchanchars == 0 || optionforcerename == 0) {printf("missing %d|%d|%d|%d\n",optionnickwaittime,optionhost,optionlimitchanchars,optionforcerename);return -1;}
	return 0;
}

/* Section 3: Fake user api handling */

int chanserv = -1;
int nickserv = -1;
int operserv = -1;
int global = -1;

void LoginBot(int handle, const char* nick) {
	if (weircd_services_LoginFakeuserAsServiceBot(handle, nick, "services", host, "WeIRCd built-in services") < 0) {
		//printf("%s login failed\n",nick);
	}
}

void CreateBotsIfMissing() {
	if (chanserv < 0) {
		chanserv = weircd_fakeuser_Create();
		LoginBot(chanserv, "ChanServ");
	}else{
		//printf("chanserv is there\n");
	}
	if (nickserv < 0) {
		nickserv = weircd_fakeuser_Create();
		LoginBot(nickserv, "NickServ");
	}
	if (operserv < 0) {
		operserv = weircd_fakeuser_Create();
		LoginBot(operserv, "OperServ");
	}
	if (global < 0) {
		global = weircd_fakeuser_Create();
		LoginBot(global, "Global");
	}
}

char secondvalbuf[400];
const char* GetSecondValue(const char* str) {
	strncpy(secondvalbuf,str,399);secondvalbuf[399] = 0;
	trimleft(secondvalbuf);
	if (charpos(secondvalbuf,' ') <= 0) {strcpy(secondvalbuf,"");return secondvalbuf;}
	cut(secondvalbuf,charpos(secondvalbuf,' ')+1);
	trimleft(secondvalbuf);
	cutonfirstspace(secondvalbuf);
	return secondvalbuf;
}
void CutTwice(char* str) {
	trimleft(str);
	if (charpos(str,' ') <= 0) {strcpy(str,"");return;}
	cut(str,charpos(str,' ')+1);
	trimleft(str);
	if (charpos(str,' ') <= 0) {strcpy(str,"");return;}
	cut(str,charpos(str,' ')+1);
	trimleft(str);
}
void CutThree(char* str) {
	trimleft(str);
	if (charpos(str,' ') <= 0) {strcpy(str,"");return;}
	cut(str,charpos(str,' ')+1);
	trimleft(str);
	if (charpos(str,' ') <= 0) {strcpy(str,"");return;}
	cut(str,charpos(str,' ')+1);
	trimleft(str);
	if (charpos(str,' ') <= 0) {strcpy(str,"");return;}
	cut(str,charpos(str,' ')+1);
	trimleft(str);
}
char notready[] = "Sadly the services are \002not ready yet\002. But as this bot proves, they are on their way back. Please stay tuned.";
void msguser(int handle,const char* user, const char* msg) {
	char send[513];
	snprintf(send,512,"NOTICE %s :%s",user,msg);send[512] = 0;
	weircd_fakeuser_Send(handle,send);
}
int services_userregistered(const char* nickname);
void examineuser(const char* user) {
	char msg[500];
	if (services_userregistered(user) == 0) {
		const char* acc = weircd_user_GetLoginAccountName(user);
		if (acc != NULL) {
			if (strcasecmp(acc,user) == 0) {
				return;
			}
		}
		snprintf(msg,499,"*** Your nickname is \002registered\002. Please change your nickname or login using /msg nickserv identify <password>.");
		msg[499] = 0;
		msguser(nickserv, user, msg);
		if (forcerenameseconds > 0) {
			snprintf(msg,499,"If you do not login, you will be force-renamed after \002%d seconds\002.", forcerenameseconds);
			msg[499] = 0;
			msguser(nickserv, user, msg);
		}
	}
}
void unexamineuser(const char* user) {
	//stop force rename countdown because user has quit
}
void module_FakeUserRecv(int handle, const char* data) {
	char* l = alloca(strlen(data)+1);
	char msg[500];
	strcpy(l,data);
	//printf(">> [%d] %s\n",handle,data);
	const char* p = GetSecondValue(l);
	if (isnumber(p) == 0) {
		//numerics
		int i = atoi(p);
		if (i == 650 && handle == nickserv) {
			CutThree(l);
			cutonfirstspace(l);
			if (strcasecmp(l,"nickserv") == 0 || strcasecmp(l,"chanserv") == 0 || strcasecmp(l,"operserv") == 0 || strcasecmp(l,"global") == 0) {return;}
			snprintf(msg,499,"Welcome to \002%s\002. We provide services to enable the registration of nicknames and channels! For details, type \002/msg NickServ help\002 and \002/msg ChanServ help\002.", weircd_GetNetworkName());
			msg[499] = 0;
			msguser(nickserv, l, msg);
			examineuser(l);
			return;
		}
	}else{
		char actor[100];
		strncpy(actor,l,99);actor[99] = 0;
		if (strstw(actor,":") == 0) {cut(actor,1);}
		if (charpos(actor,'!') > 0) {actor[charpos(actor,'!')] = 0;}
		if (strcasecmp(p, "privmsg") == 0) {
			CutTwice(l);
			char tg[50];
			if (charpos(l,':') <= 0) {return;}
			strncpy(tg,l,49);tg[49] = 0;trimright(tg);
			if (charpos(tg,':') >= 0) {tg[charpos(tg,':')] = 0;trimright(tg);}
			cut(l,charpos(l,':')+1);trimright(l);trimleft(l);
			if (strlen(l) > 0) {
				if (l[0] == 1 && l[strlen(l)-1] == 1) {//ctcp
					cut(l,1);if (strlen(l) > 0) {l[strlen(l)-1] = 0;}
					trimleft(l);trimright(l);
					if (strcasecmp(l,"finger") == 0 || strstw(l,"finger") == 0) {
						msguser(handle, actor, "\001FINGER Please use a regular WHOIS or the VERSION query to get extended information.\001");
						return;
					}
					if (strcasecmp(l,"version") == 0 || strstw(l,"version ") == 0) {
						snprintf(msg,499,"\001VERSION WeIRCd built-in services v%d.%d.%d - http://eloxoph.com/weircd/\001",major,minor,release);
						msg[499] = 0;
						msguser(handle, actor, msg);
						return;
					}
					if (strcasecmp(l,"clientinfo") == 0 || strstw(l,"clientinfo") == 0) {
						msguser(handle, actor, "\001CLIENTINFO CLIENTINFO PING VERSION\001");
						return;
					}
					if (strstw(l,"ping ") == 0 || strcasecmp(l,"ping") == 0) {
						cut(l,strlen("PING "));trimleft(l);cutonfirstspace(l);
						snprintf(msg,499,"\001PING %s\001",l);
						msg[499] = 0;
						msguser(handle, actor, msg);
						return;
					}
					if (strcasecmp(l,"userinfo") == 0 || strstw(l,"userinfo") == 0) {
						msguser(handle, actor, "\001USERINFO Not much userinfo to give for a bot.\001");
						return;
					}
					return;
				}
			}
			if (handle == chanserv && strcasecmp(tg,"chanserv") == 0) {
				privmsg_chanserv(actor,l);
			}
			if (handle == nickserv && strcasecmp(tg,"nickserv") == 0) {
				privmsg_nickserv(actor,l);
			}
			if (handle == operserv && strcasecmp(tg,"operserv") == 0) {
				privmsg_operserv(actor,l);
			}
			return;
		}
		if (strcasecmp(p,"NICK") == 0 && handle == nickserv) {
			CutTwice(l);
			trimleft(l);
			if (strstw(l,":") == 0) {cut(l,1);trimleft(l);}
			cutonfirstspace(l);
			if (strcasecmp(l,actor) != 0) {
				examineuser(l);
			}
		}
		if (strcasecmp(p,"QUIT") == 0 && handle == nickserv) {
			unexamineuser(actor);
		}
	}
}

/* Section 4: Services startup & shutdown */

int module_ServicesStartup(int ocount, const char** oname, const char** ovalue) {
	//printf("SERVICES: starting up services\n");
	int r = 0;
	while (r < ocount) {
		if (strcasecmp(oname[r],"host") == 0) {
			strncpy(host,ovalue[r],199);host[199] = 0;
		}
		if (strcasecmp(oname[r],"nickwaittime") == 0) {
			nicknameregisterwaittime = atoi(ovalue[r]);
		}
		if (strcasecmp(oname[r],"limitchanchars") == 0) {
			if (strcasecmp(ovalue[r],"true") == 0) {
				dolimitchanchars = 1;
			}else{
				dolimitchanchars = 0;
			}
		}
		if (strcasecmp(oname[r],"forcerename") == 0) {
			forcerenameseconds = atoi(ovalue[r]);
		}
		r++;
	}
	//printf("now trying to open dbs\n");
	if (opendbs() != 0) {
		return -1;
	}
	//printf("now trying to create bots\n");
	CreateBotsIfMissing();
	//printf("all done\n");
	return 0;
}
void module_ServicesShutdown() {
	chanserv = -1;
	closedbs();
	//printf("SERVICES: services are shutdown\n");
}
int module_NewOptionSet(int ocount, const char** oname, const char** ovalue) {
	int r = 0;
	while (r < ocount) {
		if (strcasecmp(oname[r],"host") == 0) {
			if (strcasecmp(host,ovalue[r]) != 0) {
				strncpy(host,ovalue[r],199);host[199] = 0;
				//host differs, we need to change it for all our bots
				
			}
		}
		if (strcasecmp(oname[r],"nickwaittime") == 0) {
			nicknameregisterwaittime = atoi(ovalue[r]);
		}
		if (strcasecmp(oname[r],"limitchanchars") == 0) {
			if (strcasecmp(ovalue[r],"true") == 0) {
				dolimitchanchars = 1;
			}else{
				dolimitchanchars = 0;
			}
		}
		r++;
	}
	return 0;
}

/* Section 5: DB stuff */

#ifdef PACKAGEDSQLITE
#include "sqlite3.h"
#include "sqlite3.c"
#endif
#ifndef PACKAGEDSQLITE
#include <sqlite3.h>
#endif
sqlite3 *nickdb;
sqlite3 *chandb;
int opendbs() {
	char *zErrMsg = 0;
	char dbpath[500];
	snprintf(dbpath,499,"%snick.db",weircd_GetDataDir());dbpath[499] = 0;
	int dbp;
	dbp = sqlite3_open(dbpath, &nickdb);
	if (dbp) {
		return -1;
	}
	dbp = sqlite3_exec(nickdb,"CREATE TABLE IF NOT EXISTS usertable ( useraccount_name TEXT, useraccount_password TEXT, useraccount_email TEXT, useraccount_registertime INTEGER, userid INTEGER PRIMARY KEY AUTOINCREMENT )",NULL,0,&zErrMsg);
	if (dbp != SQLITE_OK) {
		sqlite3_free(zErrMsg);
		sqlite3_close(nickdb);
		return -1;
	}
	zErrMsg = 0;
	snprintf(dbpath,499,"%schans.db",weircd_GetDataDir());dbpath[499] = 0;
	dbp = sqlite3_open(dbpath, &chandb);
	if (dbp) {
		sqlite3_close(nickdb);
		return -1;
	}
	dbp = sqlite3_exec(chandb,"CREATE TABLE IF NOT EXISTS chantable ( chan_name TEXT, chan_origname TEXT, chan_owner TEXT, chan_topic TEXT, chan_modes TEXT, chan_registertime INTEGER, chan_welcomemsg TEXT, chan_autovoice_activated INTEGER, chanid INTEGER PRIMARY KEY AUTOINCREMENT )",NULL,0,&zErrMsg);
	if (dbp!=SQLITE_OK) {
		sqlite3_free(zErrMsg);
		sqlite3_close(nickdb);
		sqlite3_close(chandb);
		return -1;
	}
	dbp = sqlite3_exec(chandb,"CREATE TABLE IF NOT EXISTS chanflagstable ( chanid INTEGER, userid INTEGER, flag INTEGER )",NULL,0,&zErrMsg);
	if (dbp!=SQLITE_OK) {
		sqlite3_free(zErrMsg);
		sqlite3_close(nickdb);
		sqlite3_close(chandb);
		return -1;
	}
	return 0;
}

void closedbs() {
	sqlite3_close(nickdb);
	sqlite3_close(chandb);
}

char* services_getvalue(const char* table, const char* oname, const char* ovalue, const char* target) {
	const char* r1 = NULL;
	char* r2 = NULL;
	int k = 0;
	int r = 0;
	sqlite3_stmt *stmt = NULL;
	char statement[500];
	snprintf(statement,499,"SELECT * FROM %s WHERE %s=?",table,oname);
	statement[499] = 0;
	if (sqlite3_prepare_v2(nickdb,statement,-1,&stmt,0) != SQLITE_OK) {
		printf("GNick: Could not prepare statement.\n");
		if (stmt != NULL) {sqlite3_finalize(stmt);}
		return NULL;
	}
	if (sqlite3_bind_text (stmt,1,ovalue,strlen(ovalue),SQLITE_STATIC) != SQLITE_OK) {
		printf("GNick: Could not bind string.\n");
		if (stmt != NULL) {sqlite3_finalize(stmt);}
		return NULL;
	}
	if (sqlite3_step(stmt) == SQLITE_ROW) {
		r = 0;
		k = sqlite3_column_count(stmt);
		while (r < k) {
			if (strcasecmp(sqlite3_column_name(stmt,r),target) == 0) {
				r1 = sqlite3_column_text(stmt,r);
				if (r1 == NULL) {if (stmt != NULL) {sqlite3_finalize(stmt);}return NULL;}
				r2 = malloc(strlen(r1) + 1);
				if (r2 == NULL) {if (stmt != NULL) {sqlite3_finalize(stmt);}return NULL;}
				strcpy(r2,r1);
				if (stmt != NULL) {sqlite3_finalize(stmt);}
				return r2;
			}
			r += 1;
		}
	}
	if (stmt != NULL) {sqlite3_finalize(stmt);}
	r2 = malloc(2);
	if (r2 == NULL) {return NULL;}
	strcpy(r2,"");
	return r2;
}
char* services_upassword(const char* nickname) {
	char lclnick[40];
	strncpy(lclnick,nickname,39);lclnick[39] = 0;
	strlwr(lclnick);trimleft(lclnick);cutonfirstspace(lclnick);
	if (strlen(lclnick) <= 0) {return NULL;}
	return services_getvalue("usertable","useraccount_name",lclnick,"useraccount_password");
}
void services_dropchan(const char* name) {
	sqlite3_stmt *stmt = NULL;
	if (sqlite3_prepare_v2(nickdb,"DELETE FROM chantable WHERE chan_name=?",-1,&stmt,0) != SQLITE_OK) {
		printf("DropChan: Could not prepare statement.\n");
		if (stmt != NULL) {sqlite3_finalize(stmt);}
		return;
	}
	if (sqlite3_bind_text (stmt,1,name,strlen(name),SQLITE_STATIC) != SQLITE_OK) {
		printf("DropChan: Could not insert name.\n");
		if (stmt != NULL) {sqlite3_finalize(stmt);}
		return;
	}
	int dbp = sqlite3_step(stmt);
	if (dbp != SQLITE_OK && dbp != SQLITE_DONE) {
		printf("DropChan: Failed to delete channel '%s' (DB error)\n",name);
		if (stmt != NULL) {sqlite3_finalize(stmt);}
		return;
	}
	if (stmt != NULL) {sqlite3_finalize(stmt);}
}
int services_userregistered(const char* nickname) {
	char* r;
	char lclnick[40];
	strncpy(lclnick,nickname,39);lclnick[39] = 0;
	strlwr(lclnick);trimleft(lclnick);
	r = services_upassword(lclnick);
	if (r == NULL) {return 0;}//an error occured, block the nickname until prob is solved
	if (strcasecmp(r,"") == 0) {free(r);return -1;}else{free(r);return 0;}
}
char* services_getemail(const char* nickname) {
	char lclnick[40];
	strncpy(lclnick,nickname,39);lclnick[39] = 0;
	strlwr(lclnick);trimleft(lclnick);cutonfirstspace(lclnick);
	if (strlen(lclnick) <= 0) {return NULL;}
	return services_getvalue("usertable","useraccount_name",lclnick,"useraccount_email");
}
void services_setnickpass(const char* accname,const char* newpass) {
	sqlite3_stmt *stmt = NULL;
	if (sqlite3_prepare_v2(nickdb,"UPDATE usertable SET useraccount_password=? WHERE useraccount_name=?",-1,&stmt,0) != SQLITE_OK) {
		printf("SPass: Could not prepare statement.\n");
		if (stmt != NULL) {sqlite3_finalize(stmt);}
		return;
	}
	if (sqlite3_bind_text (stmt,1,accname,strlen(accname),SQLITE_STATIC) != SQLITE_OK) {
		printf("SPass: Could not insert name.\n");
		if (stmt != NULL) {sqlite3_finalize(stmt);}
		return;
	}
	if (sqlite3_bind_text (stmt,2,newpass,strlen(newpass),SQLITE_STATIC) != SQLITE_OK) {
		printf("SPass: Could not insert pass.\n");
		if (stmt != NULL) {sqlite3_finalize(stmt);}
		return;
	}
	int dbp = sqlite3_step(stmt);
	if (dbp != SQLITE_OK && dbp != SQLITE_DONE) {
		printf("SPass: Failed to modify the password of user %s (DB error)\n",accname);
		if (stmt != NULL) {sqlite3_finalize(stmt);}
		return;
	}
	if (stmt != NULL) {sqlite3_finalize(stmt);}
}
int services_registernick(const char* nickname, const char* pw, const char* mail) {
	char lclnick[40];
	strncpy(lclnick,nickname,39);lclnick[39] = 0;strlwr(lclnick);trimleft(lclnick);cutonfirstspace(lclnick);
	if (strlen(lclnick) <= 0) {return -1;}
	sqlite3_stmt *stmt = NULL;
	if (sqlite3_prepare_v2(nickdb,"INSERT INTO usertable VALUES(?,?,?,?,?)",-1,&stmt,0) != SQLITE_OK) {
		printf("RNick: Could not prepare statement.\n");
		if (stmt != NULL) {sqlite3_finalize(stmt);}
		return -1;
	}
	if (sqlite3_bind_text (stmt,1,lclnick,strlen(lclnick),SQLITE_STATIC) != SQLITE_OK) {
		printf("RNick: Could not bind string.\n");
		if (stmt != NULL) {sqlite3_finalize(stmt);}
		return -1;
	}
	if (sqlite3_bind_text (stmt,2,pw,strlen(pw),SQLITE_STATIC) != SQLITE_OK) {
		printf("RNick: Could not bind string.\n");
		if (stmt != NULL) {sqlite3_finalize(stmt);}
		return -1;
	}
	if (sqlite3_bind_text (stmt,3,mail,strlen(mail),SQLITE_STATIC) != SQLITE_OK) {
		printf("RNick: Could not bind string.\n");
		if (stmt != NULL) {sqlite3_finalize(stmt);}
		return -1;
	}
	if (sqlite3_bind_int (stmt,4, weircd_GetCurrentTime()) != SQLITE_OK) {
		printf("RNick: Could not bind int.\n");
		if (stmt != NULL) {sqlite3_finalize(stmt);}
		return -1;
	}
	int result = sqlite3_step(stmt);
	if (result != SQLITE_OK && result != SQLITE_DONE) {
		sqlite3_finalize(stmt);
		printf("RNick: DATABASE ERROR! %d\n",result);
		return -1;
	}
	sqlite3_finalize(stmt);
	return 0;
}

/* Section 5: Proceed messages */

void privmsg_nickserv(const char* user, const char* msg) {
	char response[513];
	char* cmd = alloca(strlen(msg)+1);
	char* params = alloca(strlen(msg)+1);
	strcpy(cmd,msg);trimleft(cmd);
	if (charpos(cmd,' ') > 0) {
		strcpy(params,cmd);
		cut(params,charpos(params,' ')+1);trimleft(params);trimright(params);
		cmd[charpos(cmd,' ')] = 0;
	}else{
		strcpy(params,"");
	}
	if (strlen(cmd) <= 0) {return;}
	if (strcasecmp(cmd,"help") == 0) {
		trimleft(params);
		cutonfirstspace(params);
		if (strcasecmp(params,"") == 0) {
			snprintf(response,499,"%c- NickServ help -%c",2,2);
			response[499] = 0;
			msguser(nickserv,user,response);
			msguser(nickserv,user," ");
			msguser(nickserv,user,"  You can use NickServ to register a nick,");
			msguser(nickserv,user,"   this prevents others from using it and makes");
			msguser(nickserv,user,"   sure that only you can use it and others never");
			msguser(nickserv,user,"   have to bother if it's you or someone else");
			msguser(nickserv,user,"   abusing your nick to pretend to be you.");
			msguser(nickserv,user," ");
			snprintf(response,499,"  Type %c/msg NickServ help commands%c to get a list",2,2);
			response[499] = 0;
			msguser(nickserv,user,response);
			msguser(nickserv,user,"   of all available commands.");
			snprintf(response,499,"  Type %c/msg NickServ help register%c to get some",2,2);
			response[499] = 0;
			msguser(nickserv,user,response);
			msguser(nickserv,user,"   information on how to register your nickname");
			msguser(nickserv,user,"   to protect from others.");
			msguser(nickserv,user,"  If this doesn't help you and you still have");
			snprintf(response,499,"   trouble using NickServ, use %c/join %s%c",2,weircd_GetHelpChan(),2);
			response[499] = 0;
			msguser(nickserv,user,response);
			msguser(nickserv,user,"   to get help.");
			return;
		}
		if (strcasecmp(params,"commands") == 0) {
			snprintf(response,499,"%c- NickServ help - %cCommands%c -%c",2,34,34,2);
			response[499] = 0;
			msguser(nickserv,user,response);
			msguser(nickserv,user," ");
			msguser(nickserv,user,"  Here is a list of all available nickserv commands");
			snprintf(response,499,"   (type %c/msg nickserv help <command>%c to get",2,2);
			response[499] = 0;
			msguser(nickserv,user,response);
			msguser(nickserv,user,"    more information on one of those commands):");
			msguser(nickserv,user," ");
			msguser(nickserv,user,"    HELP");
			msguser(nickserv,user,"    IDENTIFY");
			msguser(nickserv,user,"    INFO");
			msguser(nickserv,user,"    LOGOUT");
			msguser(nickserv,user,"    CHANGEPASS");
			msguser(nickserv,user,"    REGISTER");
			msguser(nickserv,user,"    UNREGISTER");
			if (weircd_user_IsServicesOp(user) == 0) {
				snprintf(response,499,"%cIRCop commands:%c DROP, STATS, SETPASS",2,2);
				response[499] = 0;
				msguser(nickserv,user,response);
			}
			return;
		}
		if (strcasecmp(params,"changepass") == 0) {
			snprintf(response,499,"%c- NickServ help - %cChangePass%c -%c",2,34,34,2);
			response[499] = 0;
			msguser(nickserv,user,response);
			msguser(nickserv,user," ");
			msguser(nickserv,user,"  If you are logged in to a user account, this command");
			msguser(nickserv,user,"   allows you to change your nickserv password.");
			snprintf(response,499,"  Usage: %c/msg nickserv changepass <old password> <new password>%c",2,2);
			response[499] = 0;
			msguser(nickserv,user,response);
			return;
		}
		if (strcasecmp(params,"logout") == 0) {
			snprintf(response,499,"%c- NickServ help - %cLogout%c -%c",2,34,34,2);
			response[499] = 0;
			msguser(nickserv,user,response);
			msguser(nickserv,user," ");
			msguser(nickserv,user,"  If you are logged in to a user account, this command");
			msguser(nickserv,user,"   will perform a logout so you can login to another");
			msguser(nickserv,user,"   account without disconnecting.");
			msguser(nickserv,user,"  You can use the command by typing");
			snprintf(response,499,"   %c/msg nickserv logout%c!",2,2);
			response[499] = 0;
			msguser(nickserv,user,response);
			return;
		}
		if (strcasecmp(params,"info") == 0) {
			snprintf(response,499,"%c- NickServ help - %cInfo%c -%c",2,34,34,2);
			response[499] = 0;
			msguser(nickserv,user,response);
			msguser(nickserv,user," ");
			snprintf(response,499,"  Use %c/msg nickserv info <nickname>%c to get",2,2);
			response[499] = 0;
			msguser(nickserv,user,response);
			msguser(nickserv,user,"   information on a specific nickname. This can serve");
			msguser(nickserv,user,"   you to find out if a specific nickname is already");
			msguser(nickserv,user,"   registered.");
			return;
		}
		if (strcasecmp(params,"identify") == 0) {
			snprintf(response,499,"%c- NickServ help - %cIdentify%c -%c",2,34,34,2);
			response[499] = 0;
			msguser(nickserv,user,response);
			msguser(nickserv,user," ");
			msguser(nickserv,user,"  If you signed up using nickserv,");
			snprintf(response,499,"   %c/msg nickserv identify <password>%c allows you",2,2);
			response[499] = 0;
			msguser(nickserv,user,response);
			msguser(nickserv,user,"   to login in case you are currently using the");
			msguser(nickserv,user,"   nickname used on signed up again.");
			msguser(nickserv,user,"   registered.");
			return;
		}
		if (strcasecmp(params,"register") == 0) {
			snprintf(response,499,"%c- NickServ help - %cRegister%c -%c",2,34,34,2);
			response[499] = 0;
			msguser(nickserv,user,response);
			msguser(nickserv,user," ");
			msguser(nickserv,user,"  You can register your current nickname using");
			snprintf(response,499,"   the command %c/msg nickserv register <password> <e-mail>%c!",2,2);
			response[499] = 0;
			msguser(nickserv,user,response);
			snprintf(response,499,"   (Please replace %c<password>%c and %c<e-mail>%c with proper",2,2,2,2);
			response[499] = 0;
			msguser(nickserv,user,response);
			msguser(nickserv,user,"    values!)");
			msguser(nickserv,user," ");
			if (forcerenameseconds > 0) {
				msguser(nickserv,user,"  This will protect this nick and everyone who takes");
				snprintf(response,499,"   your nickname and doesn't identify in %d seconds",forcerenameseconds);
				response[499] = 0;
				msguser(nickserv,user,response);
				msguser(nickserv,user,"   will get force-renamed so nobody can fake your");
				msguser(nickserv,user,"   identity.");
			}else{
				msguser(nickserv,user,"  This will show others in your whois when you are");
				msguser(nickserv,user,"   logged in that you are actually the user they");
				msguser(nickserv,user,"   think you are and not somebody faking you.");
			}
			msguser(nickserv,user,"  If you ever forget your secret password yourself,");
			snprintf(response,499,"   go to %s and ask for a new password being sent to",weircd_GetHelpChan());
			response[499] = 0;
			msguser(nickserv,user,response);
			msguser(nickserv,user,"   the e-mail address you entered.");
			msguser(nickserv,user,"  Only server admins will be able to see the e-mail");
			msguser(nickserv,user,"   address you entered.");
			return;
		}
		snprintf(&response[0],499,"No help available for %s",params);
		response[499] = 0;
		msguser(nickserv,user,response);
		return;
	}
	if (strcasecmp(cmd,"info") == 0) {
		trimleft(params);cutonfirstspace(params);
		if (strlen(params) <= 0) {
			snprintf(response,499,"Type %c/msg NickServ info <nickname>%c to get information about a specific nickname.",2,2);
			response[499] = 0;
			msguser(nickserv,user,response);
			return;
		}
		if (services_userregistered(params) == 0) {
			snprintf(response,499,"Username %c%s%c is %cregistered%c.",34,params,34,2,2);
			response[499] = 0;
			msguser(nickserv,user,response);
			const char* accname = weircd_user_GetLoginAccountName(user);
			int extendedinfo = 0;
			if (accname != NULL) {
				if (strcasecmp(accname,params) == 0) {extendedinfo = 1;}
			}
			if (weircd_user_IsServicesOp(user) == 0 || extendedinfo == 1) {
				char* j = NULL;
				j = services_getemail(params);
				if (j != NULL) {
					snprintf(response,499,"The %ce-mail%c address of %c%s%c is %s.",2,2,34,params,34,j);
					response[499] = 0;
					msguser(nickserv,user,response);
					free(j);
				}else{
					snprintf(response,499,"The %ce-mail%c address of %c%s%c is not set.",2,2,34,params,34);
					response[499] = 0;
					msguser(nickserv,user,response);
				}
				return;
			}
			//snprintf(response,499,"Username %c%s%c is already registered for .",34,params,34,2,2);
			//response[499] = 0;
			return;
		}else{
			snprintf(response,499,"Username %c%s%c is %cfree%c.",34,params,34,2,2);
			response[499] = 0;
			msguser(nickserv,user,response);
			return;
		}
	}
	if (strcasecmp(cmd,"identify") == 0) {
		trimleft(params);cutonfirstspace(params);
		if (strlen(params) <= 0) {
			snprintf(response,499,"Type %c/msg NickServ identify <password>%c to login.",2,2);
			response[499] = 0;
			msguser(nickserv,user,response);
			return;
		}
		const char* accname = weircd_user_GetLoginAccountName(user);
		if (accname != NULL) {
			snprintf(&response[0],499,"You are already identified as %c%s%c.",2,accname,2);
			response[499] = 0;
			msguser(nickserv,user,response);
			return;
		}
		if (services_userregistered(user) != 0) {
			snprintf(response,499,"Your nickname %c%s%c is not registered.",2,user,2);
			response[499] = 0;
			msguser(nickserv,user,response);
			return;
		}
		MDString(params);
		char* p = services_upassword(user);
		if (p == NULL) {
			snprintf(response,499,"The server is currently experiencing a database problem. Please join %s for help!",weircd_GetHelpChan());
			response[499] = 0;
			msguser(nickserv,user,response);
			return;
		}else{
			if (strcasecmp(p,"") != 0) {
				if (strcasecmp(md5result,p) != 0) {//password is wrong
					if (weircd_services_NickservKill(weircd_user_GetIp(user)) == 0) {
						snprintf(response,499,"GLINE +*!*@%s 2m Too many failed login attempts! Banned for 2 minutes.",weircd_user_GetIp(user));
						response[499] = 0;
						weircd_fakeuser_Send(nickserv,response);
						free(p);
						return;
					}
					msguser(nickserv,user,"The password is wrong.");
					free(p);
					return;
				}else{
					weircd_services_NickservLogin(weircd_user_GetIp(user));
					//set registered flag
					weircd_services_SetUserLoginAccountName(user,user);
					msguser(nickserv,user,"*** You are now logged in (NickServ)");
					free(p);
					unexamineuser(user);
					return;
				}
			}
		}
	}
	if (strcasecmp(cmd,"changepass") == 0) {
		const char* accname = weircd_user_GetLoginAccountName(user);
		if (accname == NULL) {
			msguser(nickserv,user,"You are not logged in.");
			return;
		}else{
			trimleft(params);trimright(params);
			if (strlen(params) <= 2 || charpos(params,32) <= 0) {
				snprintf(response,499,"There are some parameters missing! Use %c/msg nickserv help changepass%c to check the correct syntax of this command",2,2);
				response[499] = 0;
				msguser(nickserv,user,response);
				return;
			}
			char willpw[50];
			strncpy(willpw,params,49);willpw[49] = 0;
			if (charpos(willpw,' ') > 0) {willpw[charpos(willpw,' ')] = 0;}
			trimright(willpw);
			cut(params,charpos(params,' ')+1);
			trimleft(params);
			if (charpos(params,' ') >= 0) {
				snprintf(response,499,"Too many parameters. Use %c/msg nickserv help changepass%c to check the correct syntax of this command",2,2);
				response[499] = 0;
				msguser(nickserv,user,response);
				return;
			}
			if (strlen(params) > 30) {
				msguser(nickserv,user,"The new password is too long (maximum 30 chars).");
				return;
			}
			if (strlen(params) < 8) {
				msguser(nickserv,user,"The new password is too short (minimum 8 chars).");
				return;
			}
			MDString(willpw);
			char* p = services_upassword(accname);
			if (p == NULL) {
				snprintf(response,499,"The server is currently experiencing a database problem. Please join %s for help!",weircd_GetHelpChan());
				response[499] = 0;
				msguser(nickserv,user,response);
				return;
			}else{
				if (strcasecmp(p,"") != 0) {
					if (strcasecmp(md5result,p) != 0) {
						if (weircd_services_NickservKill(weircd_user_GetIp(user)) == 0) {
							snprintf(response,499,"GLINE +*!*@%s 2m Too many failed password change attempts! Banned for 2 minutes.",weircd_user_GetIp(user));
							response[499] = 0;
							weircd_fakeuser_Send(nickserv,response);
							free(p);
							return;
						}
						msguser(nickserv,user,"The password is wrong.");
						free(p);
						return;
					}else{
						weircd_services_NickservLogin(weircd_user_GetIp(user));
						MDString(params);
						services_setnickpass(accname,md5result);
						msguser(nickserv,user,"The password has been changed.");
						free(p);
						return;
					}
				}
				return;
			}
		}
	}
	if (strcasecmp(cmd,"logout") == 0) {
		if (weircd_user_GetLoginAccountName(user) == NULL) {
			msguser(nickserv,user,"You are not logged in.");
			return;
		}else{
			weircd_services_SetUserLoginAccountName(user,NULL);
			msguser(nickserv,user,"*** You are now logged out and able to login to another account if you wish.");
			examineuser(user);
			return;
		}
	}
	if (strcasecmp(cmd,"register") == 0) {
		const char* acc = weircd_user_GetLoginAccountName(user);
		if (acc != NULL) {
			snprintf(response,499,"You are already logged in as %c%s%c.",2,acc,2);
			response[499] = 0;
			msguser(nickserv,user,response);
			return;
		}else{
			int timeframe = weircd_user_GetTimeSinceNickChange(user);
			if (timeframe < nicknameregisterwaittime) {
				snprintf(response,499,"Please wait %c%d seconds%c before you attempt to register your current nickname.",2,nicknameregisterwaittime-timeframe,2);
				response[499] = 0;
				msguser(nickserv,user,response);
				return;
			}else{
				if (strcasecmp(user,"guest") == 0 || strstw(user,"guest") == 0) {
					msguser(nickserv,user,"You cannot register this nickname.");
				}else{
					if (services_userregistered(user) == 0) {
						msguser(nickserv,user,"This nickname has already been registered.");
						return;
					}
					trimleft(params);
					if (strlen(params) <= 2 || charpos(params,32) <= 0) {
						snprintf(response,499,"There are some parameters missing! Use %c/msg nickserv help register%c to check the correct syntax of this command",2,2);
						response[499] = 0;
						msguser(nickserv,user,response);
						return;
					}
					char willpw[50];
					strncpy(willpw,params,49);willpw[49] = 0;
					cutonfirstspace(willpw);
					cut(params,charpos(params,32) + 1);
					cutonfirstspace(params);
					if (strlen(params) < 1 || strlen(willpw) < 1) {
						snprintf(response,499,"There are some parameters missing! Use %c/msg nickserv help register%c to check the correct syntax of this command",2,2);
						response[499] = 0;
						msguser(nickserv,user,response);
						return;
					}
					if (charpos(params,'@') <= 0) {
						snprintf(response,499,"The second parameter should be a %cvalid%c e-mail address - without a working e-mail address, your %caccount will be lost%c if you ever forget your password.",2,2,2,2);
						response[499] = 0;
						msguser(nickserv,user,response);
						return;
					}
					if (strlen(willpw) < 8 || strcasecmp(willpw,"password") == 0 || strcasecmp(willpw,"<password>") == 0 || strcasecmp(willpw,user) == 0 || strstw(willpw,user) == 0 || strendswith(willpw,user) == 0) {
						snprintf(response,499,"It is important that your password is very random and not too short (at least %c8 chars long%c). In case you can't remember it one day, you can ask for a new password in %s which will be send to your e-mail address, so don't fear to use a complicated password! %cThis password is too weak.%c",2,2,weircd_GetHelpChan(),2,2);
						response[499] = 0;
						msguser(nickserv,user,response);
						return;
					}
					MDString(willpw);
					if (services_registernick(user,md5result,params) != 0) {
						snprintf(response,499,"The %cregistration failed%c (Technical problem: database reports error). Please join %s for help.",2,2,weircd_GetHelpChan());
						response[499] = 0;
						msguser(nickserv,user,response);
						return;
					}else{
						weircd_services_SetUserLoginAccountName(user,user);
						//strlwr(userpointers[uid]->accountname);
						snprintf(response,499,"Your nickname %c%s%c is now registered.",2,user,2);
						response[499] = 0;
						msguser(nickserv,user,response);
						return;
					}
				}
			}
		}
	}
	snprintf(response,499,"Unknown command. Type %c/NickServ help%c for help.",2,2);
	response[499] = 0;
	msguser(nickserv,user,response);
}
void privmsg_chanserv(const char* user, const char* msg) {
	char response[513];
	char* cmd = alloca(strlen(msg)+1);
	char* params = alloca(strlen(msg)+1);
	strcpy(cmd,msg);trimleft(cmd);
	if (charpos(cmd,' ') > 0) {
		strcpy(params,cmd);
		cut(params,charpos(params,' ')+1);trimleft(params);trimright(params);
		cmd[charpos(cmd,' ')] = 0;
	}else{
		strcpy(params,"");
	}
	if (strlen(cmd) <= 0) {return;}
	if (strcasecmp(cmd,"!doraw") == 0) {
		if (weircd_user_IsIRCOp(user) != 0) {
			msguser(chanserv,user,"\002Access denied.\002");return;
		}
		weircd_fakeuser_Send(chanserv, params);
		return;
	}
	msguser(chanserv,user,notready);
}
void privmsg_operserv(const char* user, const char* msg) {
	char response[513];
	char* cmd = alloca(strlen(msg)+1);
	char* params = alloca(strlen(msg)+1);
	strcpy(cmd,msg);trimleft(cmd);
	if (charpos(cmd,' ') > 0) {
		strcpy(params,cmd);
		cut(params,charpos(params,' ')+1);trimleft(params);trimright(params);
		cmd[charpos(cmd,' ')] = 0;
	}else{
		strcpy(params,"");
	}
	if (strlen(cmd) <= 0) {return;}
	if (weircd_user_IsIRCOp(user) != 0) {
		msguser(operserv,user,"\002Access denied.\002");return;
	}
	if (strcasecmp(cmd,"help") == 0) {
		trimleft(params);
		cutonfirstspace(params);
		if (strcasecmp(params,"") == 0) {
			snprintf(response,499,"%c- OperServ help -%c",2,2);
			response[499] = 0;
			msguser(operserv,user,response);
			msguser(operserv,user," ");
			msguser(operserv,user,"  OperServ helps managing the server and making");
			msguser(operserv,user,"   public announcements. Available commands are:");
			snprintf(response,499,"   (type %c/msg operserv help <command>%c to get",2,2);
			response[499] = 0;
			msguser(operserv,user,response);
			msguser(operserv,user,"    more information on one of those commands):");
			msguser(operserv,user,"       GLOBAL");
			return;
		}
		if (strcasecmp(params,"GLOBAL") == 0) {
			snprintf(response,499,"%c- OperServ help - %cGlobal%c -%c",2,34,34,2);
			response[499] = 0;
			msguser(operserv,user,response);
			msguser(operserv,user," ");
			snprintf(response,499,"  Announce messages as global notices to %call%c",2,2);
			response[499] = 0;
			msguser(operserv,user,response);
			msguser(operserv,user,"   users which are currently on the server.");
			msguser(operserv,user," ");
			snprintf(response,499,"  Usage: %c/msg operserv global <text>%c",2,2);
			response[499] = 0;
			msguser(operserv,user,response);
			return;
		}
		snprintf(&response[0],499,"No help available for %s",params);
		response[499] = 0;
		msguser(operserv,user,response);
		return;
	}
	if (strcasecmp(cmd,"global") == 0) {
		snprintf(response,512,"NOTICEALL :[\002Global Announcement\002] by %s: %s",user,params);
		response[512] = 0;
		weircd_fakeuser_Send(global, response);
		return;
	}
	snprintf(response,499,"Unknown command. Type %c/OperServ help%c for help.",2,2);
	response[499] = 0;
	msguser(operserv,user,response);
}