//    WeIRCd IRC server software - sub file
//    Copyright (C) 2007-2009 Jonas Thiem
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License v3 as published
//    by the Free Software Foundation.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.

#include "weircd.h"

void ctracker_SetPrefixData(struct ctrackeritem* c, const char* modes, const char* symbols) {
	if (strlen(modes) != strlen(symbols)) {return;}
	if (c->prefixmodes != NULL) {
		if (strcasecmp(c->prefixmodes,modes) == 0) {return;}
	}
	if (c->prefixmodes != NULL) {free(c->prefixmodes);c->prefixmodes = NULL;}
	if (c->prefixsymbols != NULL) {free(c->prefixsymbols);c->prefixsymbols = NULL;}
	c->prefixmodes = malloc(strlen(modes)+1);
	if (c->prefixmodes == NULL) {return;}
	c->prefixsymbols = malloc(strlen(symbols)+1);
	if (c->prefixsymbols == NULL) {
		free (c->prefixmodes);c->prefixmodes = NULL;
		return;
	}
	strcpy(c->prefixmodes,modes);
	strcpy(c->prefixsymbols,symbols);
}
struct ctrackeritem* ctracker_NewItem() {
	struct ctrackeritem* c = malloc(sizeof(struct ctrackeritem));
	if (c == NULL) {return NULL;}
	memset(c,0,sizeof(struct ctrackeritem));
	ctracker_SetPrefixData(c, "vhoaq", "+%%@&~");
	return c;
}
char ctracker_GetUserChanMode(const char* user, const char* channel) {
	return 0;
}
void ctracker_FreeUser(struct ctrackeruser* n) {
	if (n->nick != NULL) {
		free(n->nick);
	}
	if (n->id != NULL) {
		free(n->id);
	}
	if (n->host != NULL) {
		free(n->host);
	}
	if (n->realname != NULL) {
		free(n->realname);
	}
	if (n->modes != NULL) {
		free(n->modes);
	}
	free(n);
}
void ctracker_FreeChannel(struct ctrackerchannel* c) {
	if (c->name != NULL) {
		free(c->name);
	}
	if (c->topic != NULL) {
		free(c->topic);
	}
	if (c->modes != NULL) {
		free(c->modes);
	}
	free(c);
}
void ctracker_FreeEventItem(struct ctrackerevent* e) {
	if (e->data1 != NULL) {free(e->data1);}
	if (e->data2 != NULL) {free(e->data2);}
	if (e->data3 != NULL) {free(e->data3);}
	if (e->data4 != NULL) {free(e->data4);}
	free(e);
}
void ctracker_FreeEventData(struct ctrackerevents* e) {
	int r = 0;
	while (r < e->eventcount) {
		ctracker_FreeEventItem(e->eventlist[r]);
		r++;
	}
	free(e->eventlist);
}
struct ctrackerevents* ctracker_Parse(struct ctrackeritem* ctracker, const char* line) {
	ctracker_FreeEventData(&ctracker->eventstorage);
	memset(&ctracker->eventstorage,0,sizeof(ctracker->eventstorage));
	//ok, now parse
}
void ctracker_Free(struct ctrackeritem* c) {
	ctracker_FreeEventData(&c->eventstorage);
	int cleandone = 0;
	while (cleandone == 0) {
		cleandone = 1;
		if (c->users != NULL) {
			ctracker_FreeUser(linkedlist_GetData(c->users));
			linkedlist_SetData(c->users,NULL);
			linkedlist_RemoveItem(&c->users,c->users);
			cleandone = 0;
		}
	}
	cleandone = 0;
	while (cleandone == 0) {
		cleandone = 1;
		if (c->channels != NULL) {
			ctracker_FreeChannel(linkedlist_GetData(c->channels));
			linkedlist_SetData(c->channels,NULL);
			linkedlist_RemoveItem(&c->channels,c->channels);
			cleandone = 0;
		}
	}
	if (c->prefixsymbols != NULL) {
		free(c->prefixsymbols);
	}
	if (c->prefixmodes != NULL) {
		free(c->prefixmodes);
	}
	if (c->ownmodes != NULL) {
		free(c->ownmodes);
	}
	if (c->ownnick != NULL) {
		free(c->ownnick);
	}
	if (c->ircd_004 != NULL) {
		free(c->ircd_004);
	}
	if (c->ircd_005 != NULL) {
		free(c->ircd_005);
	}
	free(c);
}