//    WeIRCd IRC server software - sub file
//    Copyright (C) 2007-2009 Jonas Thiem
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License v3 as published
//    by the Free Software Foundation.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.

#include "weircd.h"



#ifndef WIN

#include <sys/types.h>
#include <sys/select.h>
#include <netdb.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>

#endif



int wantsend = 0; //send counter, if <= zero we don't want to send anything


fd_set weircdselectset;
fd_set* readset;
fd_set* writeset;
int fdnr = 0;

int so_sysinited = 0;
static int soinitfunc() {
	if (so_sysinited == 1) {return 0;}
	FD_ZERO(&weircdselectset);
#ifdef WIN
	WSADATA wsa;
	long r = WSAStartup(MAKEWORD(2,0),&wsa);
	if (r != 0) {
		printf("%s    > Was not able to start winsock system!\n",startupprefixsub);
		return -1;
	}
#endif
	if (readset == NULL) {
		readset = malloc(sizeof(weircdselectset));
		if (readset == NULL) {
			printf("%s    > Was not able to start socket system!\n",startupprefixsub);
			return -1;
		}
	}
	if (writeset == NULL) {
		writeset = malloc(sizeof(weircdselectset));
		if (writeset == NULL) {
			printf("%s    > Was not able to start socket system!\n",startupprefixsub);
			return -1;
		}
	}
	FD_ZERO(writeset);FD_ZERO(readset);
	so_sysinited = 1;
	return 0;
}
void so_SetSocketNonblocking(int socket) {
#ifndef WIN
	int save_fd;
	save_fd = fcntl(socket, F_GETFL);
	save_fd |= O_NONBLOCK;
	fcntl(socket, F_SETFL, save_fd);
#endif
#ifdef WIN
	long l = 1;
	ioctlsocket(socket,FIONBIO,(unsigned long*)&l);
#endif
}
void so_ManageSocketWithSelect(int socket) {
	so_SetSocketNonblocking(socket);
	FD_SET(socket,&weircdselectset);
	if (fdnr <= socket) {fdnr = socket+1;}
	return;
}
int so_CreateSocket(int addToSelect) {
	if (soinitfunc() < 0) {return -1;}
	int newsock;
#ifdef IPV6
	newsock = socket(AF_INET6, SOCK_STREAM, 0);
#endif
#ifndef IPV6
	newsock = socket(AF_INET, SOCK_STREAM, 0);
#endif
	if (newsock <= 0) {
		return -1;
	}
	so_SetSocketNonblocking(newsock);
	if (addToSelect == 1) {so_ManageSocketWithSelect(newsock);}
	return newsock;
}
int so_MakeSocketListen(int socket, int port, const char* bindip) {
	if (soinitfunc() < 0) {return -1;}
	// ( 1 ) --- prepare address struct
#ifdef IPV6
	struct sockaddr_in6 addressstruct;
#endif
#ifndef IPV6
	struct sockaddr_in addressstruct;
#endif
	memset(&addressstruct, 0, sizeof(addressstruct));
#ifdef IPV6
	addressstruct.sin6_family = AF_INET6;
#endif
#ifndef IPV6
	addressstruct.sin_family = AF_INET;
#endif
	// ( 2 ) --- get the ip specified in "bindip" into the address struct
	if (strcasecmp(bindip,"any") != 0) {
		//1- now bind depending on operating system and ip family (ipv4 or ipv6)
		/* LINUX/UNIX */
#ifndef WIN
		#ifdef IPV6
			inet_pton(AF_INET6, bindip, &(addressstruct.sin6_addr));
		#endif
		#ifndef IPV6
			inet_pton(AF_INET, bindip, &(addressstruct.sin_addr.s_addr));
		#endif
#endif
		/* WINDOWS */
#ifdef WIN
		#ifdef IPV6
			//FIXME: provide windows equivalent here
			#error IPv6 support for windows is not available yet
		#endif
		#ifndef IPV6
			addressstruct.sin_addr.s_addr = inet_addr(bindip);
		#endif
#endif

		//2- check if binding worked
#ifdef IPV6
		if (IN6_IS_ADDR_UNSPECIFIED(&addressstruct.sin6_addr) == 1) {
			return -1;
		}
#endif
#ifndef IPV6
		if (addressstruct.sin_addr.s_addr == INADDR_ANY || addressstruct.sin_addr.s_addr == INADDR_NONE) {
			return -1;
		}
#endif
	}else{
#ifdef IPV6
		addressstruct.sin6_addr = in6addr_any;
#endif
#ifndef IPV6
		addressstruct.sin_addr.s_addr = INADDR_ANY;
#endif
	}
	// ( 3 ) --- set port
#ifdef IPV6
	addressstruct.sin6_port = htons(port);
#endif
#ifndef IPV6
	addressstruct.sin_port = htons(port);
#endif
	// allow reusing the address on unix systems
#ifndef WIN
	long lclyes = 1;
	setsockopt(socket, SOL_SOCKET,SO_REUSEADDR, &lclyes, sizeof(int));
#endif
	// ( 4 ) --- the actual binding process & listening
	if (bind(socket,(struct sockaddr *) &addressstruct,sizeof(addressstruct)) < 0) {
		return -1;
	}
	listen(socket, port);
	return 0;
}
/*const char* ResolveAddress(const char* tg) {
	
}*/
int so_ConnectSocketToIP(int socket, const char* ip) {
	//FIXME: implement this?
	return -1;
}
void so_SelectWait() {
	if (soinitfunc() < 0) {return;}
	struct timeval t;
	memset(&t,0,sizeof(struct timeval));
	t.tv_usec = 800000;
	memcpy(readset,&weircdselectset,sizeof(weircdselectset));
	memcpy(writeset,&weircdselectset,sizeof(weircdselectset));
	int r;
	//printf("DEBUG: select(): Call... (wantsend: %d)\n",wantsend);
	if (wantsend > 0) {
		r = select(fdnr,readset,writeset,NULL,&t);
	}else{
		r = select(fdnr,readset,NULL,NULL,&t);
		FD_ZERO(writeset);
	}
	if (r <= 0) {
		FD_ZERO(readset);FD_ZERO(writeset);
		//printf("DEBUG: select(): NOTHING.\n");
	}else{
		//printf("DEBUG: select(): --- Something to read/write.\n");
	}
}
void so_CloseSocket(int socket) {
	if (soinitfunc() < 0) {return;}
	FD_CLR(socket, &weircdselectset);
	if (readset != NULL) {FD_CLR(socket, readset);}
	if (writeset != NULL) {FD_CLR(socket, writeset);}
	close(socket);
}
int so_AcceptConnection(int socket, char* writeip, int* writesocket) {
	if (soinitfunc() < 0) {return -1;}
	socklen_t addrlen;
#ifdef IPV6
	struct sockaddr_in6 address;
#endif
#ifndef IPV6
	struct sockaddr_in address;
#endif
	addrlen = sizeof(address);
	int new_socket;
	if (!FD_ISSET(socket,readset)) {return -1;}
	new_socket = accept(socket,(struct sockaddr *) &address,&addrlen);
	if (new_socket > 0) {
		so_ManageSocketWithSelect(new_socket);
		//write the ip address into the "writeip" buf
		if (writeip != NULL) {
			int cnt = IPMAXLEN;
#ifdef WIN // WINDOZE
		#ifdef IPV6
			getnameinfo((struct sockaddr *)&address.sin6_addr, sizeof(struct
			sockaddr_in6), writeip, cnt, NULL, 0, NI_NUMERICHOST);
		#endif
		#ifndef IPV6
			//inet_ntop(AF_INET, &address.sin_addr, writeip, cnt);
			getnameinfo((struct sockaddr *)&address.sin_addr, sizeof(struct
			sockaddr_in), writeip, cnt, NULL, 0, NI_NUMERICHOST);
		#endif
#endif

#ifndef WIN // LINUX/POSIX
		#ifdef IPV6
			inet_ntop(AF_INET6, &address.sin6_addr, writeip, cnt);
		#endif
		#ifndef IPV6
			inet_ntop(AF_INET, &address.sin_addr, writeip, cnt);
		#endif
#endif
			writeip[IPMAXLEN] = 0;
		}
		*writesocket = new_socket;
		return 0;
	}
	return -1;
}
int so_GetIPLen() {
	return IPMAXLEN;
}
int so_CheckIp(const char* ip) { // check if an IP is an IP
	return -1;
}
int so_SelectSaysWrite(int sock) {
	if (soinitfunc() < 0) {return -1;}
	if (!FD_ISSET(sock, writeset)) {
		return -1;
	}
	return 0;
}
int so_SelectSaysRead(int sock) {
	if (soinitfunc() < 0) {return -1;}
	if (!FD_ISSET(sock, readset)) {
		return -1;
	}
	return 0;
}
int so_SendOnSocket(int sock,const char* buffer,int buffsize) {
	errno = 0;
	int retval = send(sock,buffer,buffsize,0);//MSG_NOSIGNAL);
	if (retval > 0) {
		return retval;
	}else{
#ifdef WIN
		int err = WSAGetLastError();
		if ((err != WSAEWOULDBLOCK && err != WSAEINTR && err != WSAENOBUFS) || retval == 0) {
#endif
#ifndef WIN
		if ((errno != EWOULDBLOCK && errno != EINTR && errno != ENOBUFS) || retval == 0) {
#endif
			return 0;//EOF/fatal error which leads to stream stopping
		}else{
			return -1;//just a temporary error
		}
	}
}
int so_ReceiveData(int socket, char* buf, int len) {
	int z = recv(socket,buf,len,0);
	if (z > 0) {//we received something!
		return z;
	}else{	//something happened. check if it's no EAGAIN/-WOULDBLOCK (and other temporary stuff)
#ifdef WIN
		int err = WSAGetLastError();
		if ((err != WSAEWOULDBLOCK && err != WSAEINTR && err != WSAENOBUFS) || z == 0) {
#endif
#ifndef WIN
		if ((errno != EWOULDBLOCK && errno != EINTR && errno != EAGAIN && errno != ENOBUFS) || z == 0) {
#endif
			return 0;//EOF/fatal error which leads to stream stopping
		}else{
			return -1;//just a temporary error
		}
	}
}