//    WeIRCd IRC server software - sub file
//    Copyright (C) 2007-2009 Jonas Thiem
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License v3 as published
//    by the Free Software Foundation.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.

#ifndef FIXCONF
#include "weircd.h"
#endif
#ifdef FIXCONF
#include "weircd_os.h"
#include "strings.h"
#include "creader.h"
#include "updateconf.h"
#include <stdlib.h>
#include <malloc.h>
#endif

char updateerror[500] = "";
void* contents = NULL;

int updateconf_GetVersion() {
	int maxbuflen = 300;
	char buf[300] = "";
	if (creader_Init() < 0) {
		return -1;
	}
	int c = creader_GetC();
	while (c != CONFEOF) {
		//fetch lines
		if (c != ';' && c != '\n') {
			addtobuf(buf,c,maxbuflen);
		}else{
			trimright(buf);trimleft(buf);
			if (strstw(buf,"Version=") == 0) {
				cut(buf,strlen("Version="));
				creader_Close();
				if (strcasecmp(buf,"0.7") == 0) {
					return CONFV0_7;
				}
				return CONFVBORKED;
			}
			strcpy(buf,"");
		}
		c = creader_GetC();
	}
	creader_Close();
	return CONFV0_6;
}
char* newconfbuf = NULL;
int newconfbufsize = 0;
#define NEWCONFCHUNKSIZE 1024
void cupdate_ComposeNew() {
	if (newconfbuf != NULL) {
		free(newconfbuf);
	}
	newconfbuf = malloc(NEWCONFCHUNKSIZE);
	newconfbufsize = NEWCONFCHUNKSIZE;
	strcpy(newconfbuf,"");
}
void cupdate_CAddS(const char* string) {
	while (strlen(newconfbuf)+strlen(string)+1 >= newconfbufsize) {
		newconfbufsize += NEWCONFCHUNKSIZE;
		newconfbuf = realloc(newconfbuf, newconfbufsize);
	}
	strcat(newconfbuf,string);
}
void cupdate_CAddC(char s) {
	char str[2];
	str[0] = s;str[1] = 0;
	cupdate_CAddS(str);
}
void cupdate_CDelC() {
	if (strlen(newconfbuf) > 0) {
		newconfbuf[strlen(newconfbuf)-1] = 0;
	}
}
char cupdate_CLastC() {
	if (strlen(newconfbuf) > 0) {
		return newconfbuf[strlen(newconfbuf)-1];
	}
	return 0;
}
FILE* fdesc;
int cupdate_Open(const char* filename, int write) {
	//if we have a target path, open it manually. Otherwise, have the internal
	//config reader system check the default path and open whatever seems
	//suitable
	if (filename == NULL) {
		int retval;
		if (write == 0) {
			retval = creader_Init();
		}else{
			retval = creader_WriteOpen();
		}
		if (retval < 0) {
			strcpy(updateerror,"Failed to open config file");
			return -1;
		}
		fdesc = creader_FileDesc();
		return 0;
	}else{
		if (write == 0) {
			fdesc = fopen(filename,"r");
		}else{
			fdesc = fopen(filename,"w");
		}
		if (!fdesc) {
			strcpy(updateerror,"Failed to open config file");
			return -1;
		}
		return 0;
	}
}

void cu_06to07_BracketsAfterLinebreaks(const char* filename) {
	//the new 0.7 config doesn't allow brackets to be in another line than the block name
	if (cupdate_Open(filename,0) < 0) {
		return;
	}
	int lastbreak = 0;
	int ret = fgetc(fdesc);
	while (ret != EOF) {
		char c = (char)ret;
		if (c == '\n') {
			lastbreak = 1;
		}else{
			if (c != ' ' && c != 9) {
				if (c == '{' && lastbreak == 1) {
					cupdate_CDelC();//delete last line break
					if (cupdate_CLastC() == '\r') {
						cupdate_CDelC();
					}
					if (cupdate_CLastC() != ' ') {
						cupdate_CAddC(' ');
					}
					cupdate_CAddC(c);
				}else{
					lastbreak = 0;
					cupdate_CAddC(c);
				}
			}
		}
		ret = fgetc(fdesc);
	}
	fclose(fdesc);
}
	